
#    The Circle - Decentralized resource discovery software
#    Copyright (C) 2001  Paul Francis Harrison
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Fundamental decentralized hash networking class

     Provides the basic network services that all other components of
     Circle rely on.

     - remote procedure calls with "call"
     - decentralized hashtable publishing and searching
"""

# TODO: ICMP on windows

# TODO: something goes wrong after sleeping... what?

# Problem: A node does not know its own address
#  - on receiving an address map None -> real address

# TODO:
#  - monitor messages for sanity

# Dealing with common keys 
#   - salt names with 4 bytes to make them mostly unique
#   - choose node name carefully: one of the names from last time

# For simplicity, each datum we publish is salted with the same salt

# TODO: insort peers, links, data

# TODO: do we really need large buffers?

# TODO: (toby)
#   allow server side keyword checking, rather than d/ling all then checking them client side

import sys,os,string,socket,time,select,traceback,types,random,math,signal,Queue

from settings import *
from utility import *
from safe_pickle import *
from error import Error
import proxy

if sys.platform == 'win32':
  _recv_flags = 0
else:
  _recv_flags = socket.MSG_DONTWAIT

# Check if on masq network

def need_proxy():
    """Check if Circle needs to work around masquerading by starting a proxy.
         This is called by the Circle login screen, which then decides whether
         or not to use the Proxy class."""

    if sys.platform == 'win32':
        gateway_column = 2
    else:
        gateway_column = 1

    error = 0
    try:
        #pipe = os.popen('/sbin/route -n','r')

        # netstat also available under BSD
        pipe = os.popen('netstat -rn','r')

        pipe.readline()
        while 1:
            line = pipe.readline()
            if not line:
                break

            list = string.split(line)
            if len(list) < 3:
                continue

            #if list[2] == '0.0.0.0':   # genmask == this -> default gw

            # linux2: 
            #  Destination Gateway  Genmask ...
            #  0.0.0.0     10.0.0.1 0.0.0.0
            # freebsd4:
            #  Destination Gateway  Flags, ...
            #  default     10.0.0.1 ...
            # win32:
            #  Net-addr    Netmask  Gateway   Interface  Metric
            #  0.0.0.0              

            # RFC 1918 specifies 10., 172.16., 192.168.
            # DHCP uses 169.254. (?)

            if list[2] == '0.0.0.0' or list[0] == 'default' or list[0] == '0.0.0.0':
                for prefix in [ '10.', '172.16.', '192.168.', '169.254.' ]:
                    if list[gateway_column][:len(prefix)] == prefix:
                        error = 1
        pipe.close()
    except:
        pass

    return error

    #if error:
    #  raise Error('You appear to be trying to run Circle on a masquerading network.\n\n' + \
    #              'On masquerading networks, Circle needs to run a proxy on your firewall.\n\n' + \
                #       'Type "circle --help" for more information.')

def make_timeout(window):
    """ (internal) Construct a timeout that is robust against someone fscking
            with the clock. """

    now = time.time()
    if window < 0.0: 
        window = 0.0

    # poll task can get behind the times a bit...

    return (now-window, now+window)

def is_timed_out(timeout, now=None):
    """ (internal) Check a timeout. """

    if not now:
        now = time.time()
    return now > timeout[1] or now < timeout[0] 


# Address mangle / demangle

def address_mangle(address, sender):
    """(internal) It turns out to be difficult to determine your own IP address:
         every interface has its own IP, which to choose? So what we do is
         never actually send our IP, just replace it with None. Then the
         other end (which will know our IP from the socket.recvfrom) can
         substitute it back in with address_demangle."""

    if address == sender or address == ('127.0.0.1',sender[1]):
        return None
    elif address[0] == sender[0] or address[0] == '127.0.0.1':
        return (None, address[1])
    else:
        return address

def address_demangle(address, sender):
    if address == None:
        return sender
    elif address[0] == None:
        return (sender[0], address[1])
    else:
        return address

# Long name operations

salt_min = chr(0)   * name_salt
salt_max = chr(255) * name_salt

def long_name_minus(a,b):
    result = ''
    carry  = 0
    for i in range(long_name_bytes-1,-1,-1):
        value = ord(a[i]) - ord(b[i]) - carry
        if value < 0:
            carry = 1
            value = value + 256
        else:
            carry = 0
        result = chr(value) + result

    return result

# Only meaningfull on the result of a name_minus
def long_name_compare(a, b):
    for i in range(long_name_bytes):
        if ord(a[i]) < ord(b[i]): return -1
        if ord(a[i]) > ord(b[i]): return 1
    return 0

def long_name_bracketed(a,b,c):
    """(internal) Check if b is between a and c on the circle."""

    #return long_name_compare(
    #  long_name_minus(b,a),
    #  long_name_minus(c,a)
    #) <= 0
    return a<=b<=c or c<=a<=b or b<=c<=a

def name_rotate(name, ticks):
    """(internal) To increase reliability, some modules publish to multiple 
         locations. This function generates names spaced nicely around the circle."""

    reverse = 0
    for i in range(8):
        if ticks & (1<<i):
            reverse = reverse + (128>>i)

    return chr((ord(name[0])+reverse)%256)+name[1:]


class Link_state(Synchronous):
    """(internal) This class is used by Node.call to avoid flooding addresses. 
         It limits the number of packets in flight to an address at any one time."""

    def __init__(self):
        Synchronous.__init__(self)
        #self.low_priority_lock = threading.Lock()
        self.waiters = [ ]
        self.queue   = [ ]
        self.low_priority_queue   = [ ]

        self.round_trip     = udp_default_ping_time
        self.round_trip_2   = udp_default_ping_time*udp_default_ping_time
        #self.factor         = 1.0
        #self.n_successes    = 0
        #self.n_failures     = 0

        self.window         = 2.0
        
        self.error          = 0

        self.last           = time.time()

    def wake_queuers(self):
        for i in range(int(self.window)-len(self.waiters)):
            if len(self.queue):
                self.queue.pop(0).set()
                if self.low_priority_queue:
                    self.queue.append(self.low_priority_queue.pop(0))
            elif len(self.low_priority_queue):
                self.low_priority_queue.pop(0).set()

    def error_received(self):
        self.lock.acquire()
        self.error = 1
        self.error_timeout = make_timeout(udp_error_time) #time.time() + udp_error_time
        #self.n_successes = 0
        #self.n_failures = 0
        for event in self.waiters:
            event.set()
        self.lock.release()

    def is_error(self):
        self.lock.acquire()
        try:
            return self.error and not is_timed_out(self.error_timeout)
        finally:
            self.lock.release()

    def acquire_access(self, event, low_priority=0):
        #if low_priority:
        #  self.low_priority_lock.acquire()

        wait_event = threading.Event()
        self.lock.acquire()
        if low_priority:
            self.low_priority_queue.append(wait_event)
        else:
            self.queue.append(wait_event)
        self.wake_queuers()
        self.lock.release()
        wait_event.wait()

        self.lock.acquire()
        if self.error and not is_timed_out(self.error_timeout):
            #if low_priority:
            #  self.low_priority_lock.release()
            self.wake_queuers() 
            self.lock.release()
            return None

        self.waiters.append(event)

        #wait_time = self.round_trip*2.0
        s_dev = self.round_trip_2 - self.round_trip*self.round_trip
        if s_dev < 0.0:
            s_dev = 0.0
        else:
            s_dev = math.sqrt(s_dev)

        max_round_trip = self.round_trip + 2.0*s_dev

        if max_round_trip < udp_min_ping_time:
            max_round_trip = udp_min_ping_time
        if max_round_trip > udp_max_ping_time:
            max_round_trip = udp_max_ping_time
        
        self.lock.release()

        return max_round_trip

        #print self.round_trip, s_dev, '         ', max_round_trip
        
        #if max_round_trip < udp_min_ping_time:
        #  max_round_trip = udp_min_ping_time
        #wait_time = max_round_trip*self.factor
        #if wait_time > udp_max_ping_time:
        #  wait_time = udp_max_ping_time
        #  self.factor = wait_time/max_round_trip
        #  if self.factor < 1.0:
        #    self.factor = 1.0
        #self.lock.release()
        # 
        # wait_time = max_round_trip

        # return wait_time

    def release_access(self, event, success, n_failures, time_taken, low_priority=0):
        # reply is very likely from first Tx
    
        self.lock.acquire()
        if success:
            self.round_trip   = self.round_trip*0.9+time_taken*0.1
            self.round_trip_2 = self.round_trip_2*0.9+time_taken*time_taken*0.1

            if len(self.waiters) == int(self.window):
                self.window = self.window + 0.5/self.window
                if self.window > udp_max_window:
                    self.window = udp_max_window

        for i in range(n_failures):
            self.window = self.window - 0.5
            if self.window < 1.0:
                self.window = 1.0

        self.waiters.remove(event)

        self.last = time.time()
        
        self.wake_queuers()
        
        self.lock.release()

class Peer:
    """(internal) This structure is used by Node to store information about the 
         peers that it currently knows about.""" 

    def __init__(self, name, address):
        self.name    = name
        self.address = address

        self.poll_interval = peer_poll_start_interval
        self.timeout       = make_timeout(self.poll_interval) #time.time() + self.poll_interval

class Link:
    """(internal) This structure is used to store information about links that 
         the Node is in charge of."""

    def __init__(self, name,dest):
        self.name    = name
        self.dest    = dest

        self.poll_interval = start_poll_interval * (random.random()+0.5)
        self.timeout       = make_timeout(self.poll_interval) #time.time() + self.poll_interval

def poll_link_task(node,link, old_timeout):
    #print "Polling %s" % nickname(link.name)

    ok = 1

    # Drop inappropriate links (eg if a new peer has come online)
    node.lock.acquire()
    right = node.right_neighbours[:1]
    if right:
        right_address = node.peers[right[0]].address
    node.lock.release()
    if right and \
         long_name_bracketed(node.name, right[0], link.name):
        ok = 0
    
    #if link.dest == None:
    #  address = node.address
    #else:
    #  address = link.dest
    address = address_demangle(link.dest, node.address)
        
    if ok:
        try:
            node.call(address,('poll',link.name,int(link.poll_interval + poll_breathingspace)), 1)
        except Error, error:
            #ok = 0
            node.lock.acquire()
            if link in node.links:
                node.links.remove(link)

            if error.message == 'no reply':
                i = 0
                while i < len(node.links):
                    if node.links[i].dest == link.dest:
                        del node.links[i]
                    else:
                        i = i + 1
            node.lock.release()
    else:
        try:
            try:
                if not right:
                    raise Error('no right neighbour')

                node.call(right_address,('store link for',link.name,link.dest,int(old_timeout[1]-time.time())), 1)
            except Error:
                node.call(address,('dropped link',link.name), 1)
        except Error:
            #print 'Attempt to inform peer of dropped link failed'
            pass

        #if not ok:
        node.lock.acquire()
        if link in node.links:
            node.links.remove(link)
        node.lock.release()

def poll_peer_task(node, peer):
    ok = 1
    node.lock.acquire()
    if len(node.peers) > peer_cache_size and \
         peer.name not in node.left_neighbours and \
         peer.name not in node.right_neighbours:
        ok = 0
    node.lock.release()

    if ok:
        try:
            name = node.call(peer.address,('who are you',), 1)
            if name <> peer.name:
                ok = 0
        except Error:
            ok = 0

    if not ok:
        #print "Dropping peer"
        node.lock.acquire()
        if node.peers.has_key(peer.name):
            del node.peers[peer.name]
        node.build_neighbours()
        node.lock.release()

def poll_task(node):
    right_name = None
    time_now = time.time()

    while 1:
        tasks = [ ]

        node.lock.acquire()
        try:
            if not node.running:
                break

            if not node.hashtable_running and not node.hashtable_stopped \
                 and time_now > node.start_time + hashtable_activate_time:
                node.lock.release()
                try:
                    #if node.ping_count and \
                    #  node.ping_total_time / node.ping_count > hashtable_activate_threshhold:
                    if node.pings_under_threshhold < node.pings_over_threshhold:    
                        node.deactivate_hashtable()
                    else:
                        node.activate_hashtable()
                finally:
                    node.lock.acquire()
     
            # Drop bad loopback links

            ## Somewhat inefficient
            #for link in node.links[:]:
            #  if link.dest == None and \
            #     len(node.right_neighbours) and \
            #     long_name_bracketed(node.name,node.right_neighbours[0],link.name):
                    #node.links.remove(link)
                    #if node.data.has_key(link.name):
                    #  node.data_timeout[link.name] = time_now-1
                    
            # Poll links
            list = [ ]

            # 28/12/01 not sure why i forgot this bit...
            if len(node.right_neighbours) and \
                 node.right_neighbours[0] <> right_name:
                right_name = node.right_neighbours[0]
                
                for link in node.links[:]:
                    if long_name_bracketed(node.name,right_name,link.name):
                        node.links.remove(link)
                        list.append(link)
     
            for link in node.links:
                if is_timed_out(link.timeout,time_now):
                    list.append(link)
        finally:
            node.lock.release()

        for link in list:
            old_timeout = link.timeout
            link.poll_interval = link.poll_interval * poll_expand_factor
            link.timeout       = make_timeout(link.poll_interval) #time.time() + link.poll_interval
            tasks.append(Task(poll_link_task,node,link,old_timeout))

        # Poll peers

        node.lock.acquire()
        try:
            list = [ ]
            for peer in node.peers.values():
                if is_timed_out(peer.timeout,time_now):
                    list.append(peer)
        finally:  
            node.lock.release()

        for peer in list:
            peer.poll_interval = peer.poll_interval * peer_poll_expand_factor
            peer.timeout       = make_timeout(peer.poll_interval) #time.time() + peer.poll_interval
            tasks.append(Task(poll_peer_task,node,peer))
        
        # Poll data

        node.lock.acquire()
        try:
            for item in node.data_timeout.items():
                if is_timed_out(item[1],time_now):
                    node.data_timeout[item[0]] = make_timeout(publish_retry_interval) #time.time() + publish_retry_interval
                    
                    def announce_task(node, item):
                        try:
                            peer = node.find_node(item, 1)
                            poll_interval = node.call(peer[1],('store link',item), 1)
                            check(poll_interval, 0)
                            timeout = make_timeout(poll_interval)
                            node.lock.acquire()
                            node.data_timeout[item] = timeout #time.time() + poll_interval
                            node.lock.release()
                        except Error, error:
                            pass
                        #node.announce(item)
                    tasks.append(Task(announce_task,node,item[0]))

            # Drop unused link states
        
            for pair in node.link_states.items():
                if pair[1].last + link_state_timeout < time_now:
                    del node.link_states[pair[0]]

        finally: 
            node.lock.release()
    
        if not tasks:
            time.sleep(5)
            time_now = time.time()
            continue
        
        time_now = time.time()

        for task in tasks:
            task.start_no_overload(1)

        while not node.task_queue.empty():
            time.sleep(0.5)

        #print len(tasks)
        # Try to avoid a new thread mostly

        #while len(tasks) > 1 and node.running:
        #  while node.overloaded():
        #    time.sleep(0.05)
        #  tasks.pop().start()

        #if node.running:
        #  apply(tasks[0].function,(tasks[0].manager,)+tasks[0].parameters)

def probe_task(node,address):
    try:
        name = node.call(address, ('who are you',))
        node.add_peer(name,address)

        #Done by add peer if necessary
        #probe_neighbours_task(node, address)
    except Error:
        pass

def probe_neighbours_task(node,address):
    try:
        try:
            list = node.call(address, ('get peers','<',node.name, neighbourhood_size))
            for i in list:
                try:
                    check(i, ('',))
                except:
                    continue
                node.add_peer(i[0],address_demangle(i[1],address), 0)

            list = node.call(address, ('get peers','>',node.name, neighbourhood_size))
            for i in list:
                try:
                    check(i, ('',))
                except:
                    continue
                node.add_peer(i[0],address_demangle(i[1],address), 0)
        finally:
            node.lock.acquire()
            node.active_probes = node.active_probes - 1
            node.lock.release()
    except Error:
        #traceback.print_exc()
        #print "Probe neighbours failed"
        pass


def handle_request(node, request, address,call_id):
    try:
        return node.reply_cache[(address,call_id)]
    except KeyError:
        try:
            result = perform_request(node,request,address,call_id)
        except Error, error:
            result = error

        node.reply_cache[(address,call_id)] = result
        node.reply_cache_keys.append((address,call_id))
        while len(node.reply_cache_keys) > reply_cache_size:
            key = node.reply_cache_keys.pop(0)
            try:
                del node.reply_cache[key]
            except KeyError:
                pass
        return result

def perform_request(node, request, address,call_id):
    check_type(request[0],types.StringType)

    if address == None:
        raise "blarg"

    # Multiple requests per packet
    if request[0] == 'glob':
        result = [ ]
        for item in request[1:]:
            result.append(perform_request(node,item,address,call_id))
        return result

    if request[0] == 'who are you':
        if not node.hashtable_running:
            return Error('hashtable not active')
        return node.name

    if request[0] == 'add peer':
        check_is_long_name(request[1])

        peer_address = address_demangle(request[2],address)
        node.add_peer(request[1], peer_address)

        return None

    if request[0] == 'get peers':
        check_type(request[1], types.StringType)
        check_is_long_name(request[2])
        check_type(request[3], types.IntType)
    
        node.lock.acquire()
        list = node.query_peers(request[2],request[1],request[3])
        result = [ ]
        for item in list:
            result.append((item,address_mangle(node.peers[item].address,node.address)))
        node.lock.release()

        return result

    # Fetch neighbours who could also be holding a particular link
    if request[0] == 'get cohorts':
        check_is_name(request[1])

        min = request[1] + salt_min
        max = request[1] + salt_max

        node.lock.acquire()
        cohorts = [ ]

        list = [(node.name,node.address)]
        for peer in node.left_neighbours:
            list.insert(0, (peer,node.peers[peer].address))
        for peer in node.right_neighbours:
            list.append((peer,node.peers[peer].address))

        for i in range(len(list)-1):
            if list[i][0] <> node.name and \
                 ( long_name_bracketed(list[i][0], min, list[i+1][0]) or \
                     long_name_bracketed(list[i][0], max, list[i+1][0]) or \
                     long_name_bracketed(min, list[i][0], max) ):
                cohorts.append(list[i])
        node.lock.release()

        return cohorts
        
    if request[0] == 'store link' or request[0] == 'store link for':
        node.lock.acquire()
        try:
            if not node.hashtable_running:
                return Error('hashtable not active')

            check_is_long_name(request[1])

            # ('store link for ', name, [address, timeout])
            if request[0] == 'store link for':
                address  = address_demangle(request[2], address)
                interval = request[3]
            else:
                interval = None

            # Special protocol for self-links, as we do not know our own address
            address = address_mangle(address, node.address)
    
            link = None
            for item in node.links:
                if item.name == request[1] and item.dest == address:
                    link = item

            if not link:
                link = Link(request[1],address)
                if interval <> None:
                    link.timeout = make_timeout(interval) #time.time() + interval
                node.links.append(link)
            else:
                link.timeout = make_timeout(link.poll_interval) #time.time() + link.poll_interval
        finally:
            node.lock.release()

        #return int(link.poll_interval + poll_breathingspace)
        return int(link.timeout[1] - time.time() + poll_breathingspace)
        
    if request[0] == 'query link':
        node.lock.acquire()
        try:
            if not node.hashtable_running:
                return Error('hashtable not active')
        
            check_is_name(request[1])

            list = [ ]
            for item in node.links:
                if item.name[:name_bytes] == request[1]:
                    list.append(item.dest)

            return list[ request[2]:request[3] ]
        finally:
            node.lock.release()
    
    if request[0] == 'query data':
        check_is_name(request[1])
    
        node.lock.acquire()
        try:
            data = node.data.get(request[1] + node.salt,[ ])
        finally:
            node.lock.release()

        result = [ ]
        for item in data[ request[2]:request[3] ]:
            if type(item) in [types.FunctionType, types.MethodType]:
                result.append(apply(item,()))
            else:
                result.append(item)

        return result

    if request[0] == 'poll':
        check_is_long_name(request[1])
        check_type(request[2],types.IntType)

        node.lock.acquire()
        #ok = request[1][name_bytes:] == node.salt and \
        #     node.data.has_key(request[1][:name_bytes])
        ok = node.data.has_key(request[1])
        if ok:
            node.data_timeout[request[1]] = make_timeout(request[2]) #time.time() + request[2]
        
        node.lock.release()

        return ok

    if request[0] == 'dropped link':
        check_is_long_name(request[1])

        # Poll task will republish
        node.lock.acquire()
        ok = request[1][name_bytes:] == node.salt and \
                 node.data.has_key(request[1][:name_bytes])
        if ok:
            node.data_timeout[request[1]] = make_timeout(0.0) #time.time()
        node.lock.release()
        
        return None

    if request[0] == 'going offline': # actually means hashtable deactivate
        node.lock.acquire()
        for item in node.peers.items():
            if item[1].address == address:
                del node.peers[item[0]]
                node.build_neighbours()
        node.lock.release()

        return None

    if node.handlers.has_key(request[0]):
        return node.handlers[request[0]].handle(request,address,call_id)

    return Error('Request not understood: '+request[0])

def handle_message(node, message, address):
    message_len = len(message)
    message = safe_loads(message)
            
    if type(message) == types.TupleType and \
         len(message) >= 3 and \
         message[0] == 'reply':
        node.lock.acquire()
        
        # Reply too late
        if not node.pending_calls.has_key(message[1]):
            node.lock.release()
        else: 
            event = node.pending_calls[message[1]]
            
            node.replies[message[1]] = (message[2], message_len)
            node.lock.release()
            event.set()

        return
                
    #if replies_only:
    #  node.socket_sendto(safe_dumps(('reply',message[1],Error('Passive peer.'))),address)
    #  return

    if message[0] <> None:
        check_is_long_name(message[0])
        node.add_peer(message[0],address)

    check_type(message[1], types.IntType)

    #print 'handling', message[2][0]

    try:
        result = handle_request(node, message[2], address,message[1])
    except:
        traceback.print_exc()
        result = Error('Bug in remote peer.')
    
    #print 'handled ', message[2][0]

    result = safe_dumps(('reply',message[1],result))
            
    node.socket_sendto(result,address)

    node.lock.acquire()
    node.network_usage = node.network_usage + len(result)
    node.lock.release()

    try:
        for monitor in node.monitors:
            monitor( 0, address, message[2], len(result)+message_len )
    except:
        traceback.print_exc()

def listen_task(node):
    def message_handler(node,queue):
        while 1:
            item = queue.get()
            if not item:
                return
            try:
                apply(handle_message,item)
            except:
                traceback.print_exc()

    queue = Queue.Queue(0)
    Task(message_handler,node,queue).start()

    while 1:
        if not node.running:
            break

        if not node.socket_proxy:
            try:
                # Signal can cause error here (eg ^C, SIGTERM)
                result = select.select([node.socket],[ ],[node.socket],2)
            except:
                result = [[],[],[]]

            if result[2]:
                node.socket_handle_errors()

            if result[0] == [ ]:
                continue
        
            usage = 0
            while 1: 
                try:
                    message, address = node.socket_recvfrom()
                except:
                    break

                usage = usage + len(message)

                queue.put((node,message,address))

                if not select.select([node.socket],[ ],[ ],0)[0]:
                    break

            node.lock.acquire()
            node.network_usage = node.network_usage + usage
            node.lock.release()

        else:
            try:
                message, address = node.socket_recvfrom()
            except Error:
                continue
            except:
                break
                
            queue.put((node,message,address))
            
            node.lock.acquire()
            node.network_usage = node.network_usage + len(message)
            node.lock.release()

    queue.put(None)
            

def node_search_task(pipe, node, search_for, redundancy, addresses_only):
    pipe.tried_addresses = [ ]

    node.lock.acquire()
    try:
        if node.data.has_key(search_for+node.salt):
            node_search_subsubtask(pipe,node,search_for,None,node.address,addresses_only)
    finally:
        node.lock.release()

    for tick in range(redundancy):
        Task(node_search_subtask,pipe,node,name_rotate(search_for,tick),addresses_only).start()

def node_search_subtask(pipe,node,search_for, addresses_only):
    salted_search = search_for
    for i in range(name_salt):
        salted_search = salted_search + chr(random.randint(0,255)) 

    try:
        peer = node.find_node(salted_search)
    except Error:
        return

    tried_peers = [ ]
    suggested_peers = [ peer ]

    while pipe.running and suggested_peers:
        peer = suggested_peers.pop()

        if peer not in tried_peers:
            tried_peers.append(peer)

            position = 0
            failed   = 0
            while pipe.running:
                try:
                    links = node.call(peer[1],('query link',search_for,position,position+20))
                except Error:
                    failed = 1
                    break

                for link in links:
                    Task(node_search_subsubtask,pipe,node,search_for,link,peer[1],addresses_only).start()

                if len(links) < 20:
                    break

                position = position + len(links)
                
            if not failed:
                try:
                    peers = node.call(peer[1],('get cohorts',search_for))
                    suggested_peers.extend(peers)
                except Error:
                    pass

def node_search_subsubtask(pipe,node,search_for,link,source,addresses_only):
    link = address_demangle(link, source)

    pipe.lock.acquire()
    try:
        if link in pipe.tried_addresses:
            return
        pipe.tried_addresses.append(link)
    finally:
        pipe.lock.release()

    if addresses_only:
        pipe.write(link)
        return

    data_position = 0
    while pipe.running:
        try:
            data = node.call(link,('query data',search_for,data_position,data_position+1) )
            check(data, [ ])
        except Error:
            break

        for datum in data:
            pipe.write((link, datum))

        if len(data) < 1:
            break

        data_position = data_position + len(data)

class Null_proxy:
    def __init__(self):
        self.queue = Queue.Queue(0)

    def sendto(self, message, address):
        self.queue.put((1,address,message))

    def recvfrom(self):
        gotten = self.queue.get()

        if not gotten:
            raise Error('stopped')

        return gotten

    def stop(self):
        self.queue.put(None)

class Node(Task_manager):
    """This class provides all of the lowest level network services in circle.
         
         - remote procedure calls using UDP (call)
         - publishing data to the hashtable (publish)
         - retrieving data from the hashtable (retrieve)
         - passing on RPC calls to other classes (add/remove_handler)"""

    def __init__(self):
        Task_manager.__init__(self)

        self.start_time       = time.time()
        self.network_usage    = 0L # This can overflow if its just an int (!)
        self.active_probes    = 0
    
        self.call_no          = 0
        self.pending_calls    = { } 
        self.replies          = { } # Call no -> (reply, reply_length)

        self.handlers         = { }
        self.reply_cache      = { } # Cache replies to incoming requests
        self.reply_cache_keys = [ ]

        self.monitors         = [ ] # For monitoring incoming requests (eg to collect stats)
    
        self.peers            = { } # Peers indexed by long name
        self.link_states      = { } # Windowing, round trip time, etc
        self.left_neighbours  = [ ]
        self.right_neighbours = [ ]
        self.data             = { } # Data indexed by long name
        self.data_timeout     = { }
        self.links            = [ ] 

        # How often are we dropping packets?
        self.call_p_success   = 1.0

        # Monitor ping times (to decide whether we should activate the hashtable)
        # (ping time for find_nodes is measured)
        #self.ping_total_time  = 0.0
        #self.ping_count       = 0
        self.pings_under_threshhold = 0
        self.pings_over_threshhold = 0

        self.hashtable_running = 0
        self.hashtable_stopped = 0
        self.hashtable_offloaders = [ ] # List of tasks offloading hashtable entries (for asynchronous hashtable shutdown)

        self.name = get_config("node_name_seed","")

        # Ensure name is not re-used, eg by daemon
        set_config("node_name_seed","")

        self.name = self.name + random_bytes(long_name_bytes - len(self.name))

        self.salt = self.name[-name_salt:]

    def save_seed(self):
        """In order to match the distribution of links about the circle, Node
             needs to choose a random link to serve as its address next time it 
             starts. This allows the circle to deal with very common links gracefully,
             eg "mp3" "service data cache".

             This should be called on shutdown, before shutting down any classes that
             provide circle services."""
        self.lock.acquire()
        if self.data:
            set_config("node_name_seed", random.choice(self.data.keys())[:name_bytes])
        self.lock.release()

    def start(self, proxy_host=None, proxy_password=None, proxy_transient=None, no_connect=0):
        """Start the Node.

             This can raise an Error if starting the proxy fails."""

        Task_manager.start(self)

        self.socket_lock = threading.Lock()

        if no_connect:
            self.socket_proxy = Null_proxy()
            self.address = ('127.0.0.1',29610)
        elif proxy_host:
            self.socket_proxy = proxy.Proxy(proxy_host,proxy_password,proxy_transient)
            self.address = self.socket_proxy.address
        else:
            self.socket_proxy = None
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

            for port in default_ports:
                try:
                    self.socket.bind(('',port))
                    break
                except:
                    pass
            else:
                self.socket.bind(('',socket.INADDR_ANY))

            self.address = (
                '127.0.0.1',
                self.socket.getsockname()[1]
            )

            # On linux, really only gives 1<<17 bytes buffers, but we can wish...
            # Note: BSD throws error instead of silently failing
            for i in range(20,12,-1):
                try:
                    self.socket.setsockopt(socket.SOL_SOCKET,socket.SO_SNDBUF,1<<i)
                    self.socket.setsockopt(socket.SOL_SOCKET,socket.SO_RCVBUF,1<<i)
                    break
                except socket.error:
                    pass

            #self.socket.setsockopt(socket.SOL_SOCKET,socket.SO_SNDBUF,1<<13)
            #self.socket.setsockopt(socket.SOL_SOCKET,socket.SO_RCVBUF,1<<13)
            
            #print self.socket.getsockopt(socket.SOL_SOCKET,socket.SO_SNDBUF)
            #print self.socket.getsockopt(socket.SOL_SOCKET,socket.SO_RCVBUF)

            # Enable ICMP error reception (SOL_IP,IP_RECVERR)
            #print self.socket.getsockopt(0,11)
            
            # Not sure how to do this on windows
            if sys.platform <> 'win32':
                self.socket.setsockopt(socket.SOL_IP,11,1)

            # Disable path MTU discovery
            # Do we need this for BSD as well (or Windows...)?
            if sys.platform == 'linux2':
                self.socket.setsockopt(socket.SOL_IP,10,0)

            #print self.socket.getsockopt(0,11)

        Task(listen_task,self).start()
        Task(poll_task,self).start()
        self.greet_known_peers()

    def stop(self):
        # Save peers for next time
        self.lock.acquire()
        try:
            known_peers = get_config("known_peers", [ ])
            for peer in self.peers.values():
                if peer.address in known_peers:
                    known_peers.remove(peer.address)
                if peer.address[1] == default_ports[0]:
                    known_peers.insert(0,peer.address)

            set_config("known_peers", known_peers[:max_known_peers])
        finally:
            self.lock.release()

        self.deactivate_hashtable()

        for task in self.hashtable_offloaders:
            task.join()
            
        self.lock.acquire()
        self.running = 0

        # Abort all pending calls
        for event in self.pending_calls.values():
            event.set()
        
        self.lock.release()
        
        if self.socket_proxy:
            self.socket_proxy.stop()
        
        Task_manager.stop(self)

    def greet_known_peers(self): 
        """ (Internal) Greet peers in initial list or saved from last time.
                Called by start. """
        known_peers = get_config("known_peers", [ ])
        any = 0
        for item in known_peers:
            # Don't allow probing loopback, as will record wrong address
            if item <> '127.0.0.1':
                self.probe(item)
                any = 1

        if not any:
            for item in initial_peers:
                try:
                    address = socket.gethostbyname(item)
                    self.probe((address,default_ports[0]))
                    any = 1
                except:
                    pass

        return any

    def is_connected(self):
        if self.peers:
            return 1
        else:
            return 0

    def add_handler(self, what, how):
        """Add a handler for a specific message.

             what: the message name (request[0])
             how : a class instance with a method defined thus:
                     handle(self, request, address, call_id)
                 where
                     request is the actual message
                     address is the address of the caller
                     call_id is (with address) a unique identifier of the message
                 and returning the reply.

                 Note: because UDP is an unreliable protocol, the handler might
                     be called more than once for the one message (if the reply
                     packet was dropped). Use (address,call_id) to check for repeats
                     if this is a problem.
                 """
        self.handlers[what] = how

    def remove_handler(self, what):
        """Remove a previously installed message handler."""
        self.lock.acquire()
        if self.handlers.has_key(what):
            del self.handlers[what]
        self.lock.release()

    def add_monitor(self, monitor):
        self.monitors.append(monitor)

    def remove_monitor(self, monitor):
        self.lock.acquire()
        if monitor in self.monitors:
            self.monitors.remove(monitor)
        self.lock.release()

    def publish(self, name, info, redundancy=1):
        """Publish a data item to the hashtable.

             name is the name of the link, it should be a string name_bytes long.
                 This is generally an MD5 hash generated by hash.hash_of.
                 eg hash.hash_of("myservice blah garfunkle")
                 
             info the data to be associated with the name. This should be a fairly
                 small dictionary, containing at least a 'type' attribute.

                 info can also be a function taking no parameters. If you use this
                 option, the function should return the data you want associated
                 with the name. This is useful for example if you want to include
                 a constantly changing value, such as up-time.
                 
             redundancy is the degree of redundancy with which to publish the data.
                 a redundancy of about 4 should guarantee that the published item is
                 always visible."""
    
        # Announcing now done by poll task
        for tick in range(redundancy):
            rot_name = name_rotate(name,tick) + self.salt

            new_key = 0

            self.lock.acquire()

            if not self.data.has_key(rot_name):
                self.data[rot_name] = [ ]
                new_key = 1

            if info not in self.data[rot_name]:
                self.data[rot_name].append(info)

            if new_key:
                self.data_timeout[rot_name] = make_timeout(0.0) #time.time()# + publish_retry_interval

            self.lock.release()

                #self.announce(rot_name)

    def unpublish(self, info):
        """Unpublish a previously published item of information.
             This will unpublish *all* links to the item.
             
             item must be the same object, not just equal."""
        self.lock.acquire()
        try:
            for name in self.data.keys():
                pos = 0
                while pos < len(self.data[name]):
                    if self.data[name][pos] is info:
                        del self.data[name][pos]
                    else:
                        pos = pos + 1

                #if info in self.data[name]:
                #  self.data[name].remove(info)

                if self.data[name] == [ ]:
                    del self.data[name]
                    del self.data_timeout[name]
        finally:
            self.lock.release()

    def retrieve(self, name, redundancy=1, addresses_only=0):
        """Retrieve data items associated with a name from the hashtable.
             This returns a pipe object that emits (address, data) pairs.

             Example usage:

             pipe = node.retrieve(hash.hash_of("thingy"))
             try:
                 while not pipe.finished():
                     for item in pipe.read_all()
                         # item[0] is the address
                         # item[1] is the data
                         ... do something ...
                     time.sleep(1)
             finally:
                 pipe.stop()
                 
             And yes, a pipe implementation that doesn't need polling would
             be nice..."""
                 
        pipe = Pipe()
        pipe.start(node_search_task,self,name,redundancy,addresses_only)
        return pipe

        #node = self.find_node(name)
        ##print "Retrieving from %s" % nickname(node[0])

        ## Todo: downloading large lists in multiple parts
        #links = self.call(node[1],('query link',name,0,20))

        #result = [ ]
        #for link in links:
        #  # Todo: downloading large lists in multiple parts
        #  list = self.call(link,('query data',name,0,20) )
        #  for item in list:
        #    result.append( (link, item) )

        #return result

    def probe(self, address):
        #self.lock.acquire()
        #self.active_probes = self.active_probes + 1
        #self.lock.release()
        Task(probe_task,self,address).start_no_overload(1)

    def probe_neighbours(self, address):
        self.lock.acquire()
        self.active_probes = self.active_probes + 1
        self.lock.release()
        Task(probe_neighbours_task, self,address).start_no_overload()

    # Internal

    #def announce(self, name):
    #  # Keep trying until you find a live one
    #  try:
    #    nodes = self.find_nodes(name, 5, 1)
    #    while len(nodes):
    #      node = nodes[0]
#       nodes = nodes[1:]
#        try:
#         poll_interval = self.call(node[1],('store link',name), 1)
#         break
#       except:
#         pass
#      else:
#        raise Error('failed')
#
#      self.lock.acquire()
#      self.data_timeout[name] = time.time() + poll_interval
#      self.lock.release()
#    except Error:
#      #print 'Publish attempt failed, will retry later'
#      pass

    def build_neighbours(self):
        self.lock.acquire()
        
        old_neighbours = self.left_neighbours + self.right_neighbours
        
        self.left_neighbours  = self.query_peers(self.name,'<',neighbourhood_size)
        self.right_neighbours = self.query_peers(self.name,'>',neighbourhood_size)

        new_neighbours = [ ]
        for peer in self.left_neighbours + self.right_neighbours:
            if peer not in new_neighbours and peer not in old_neighbours:
                new_neighbours.append(peer)

        for peer in new_neighbours:
            #print "%s: New neighbour: %s" % (nickname(self.name), nickname(peer))
            #Task(probe_neighbours_task, self,self.peers[peer].address).start()
            self.probe_neighbours(self.peers[peer].address)
        
        self.lock.release()

    # Knowledge base maintenance
    
    def add_peer(self, name, address, confirmed=1):
        if type(name) <> types.StringType or \
             self.peers.has_key(name) or name == self.name:
            return

        if not confirmed:
            try:
                name = self.call(address, ('who are you',))
                check_type(name, types.StringType)
                confirmed = 1
            except Error:
                #print "Bad peer"
                return

        self.lock.acquire()
        try:
            if self.peers.has_key(name) or name == self.name:
                return

            self.peers[name] = Peer(name,address)

            #Optimize!!
            self.build_neighbours()
        finally:
            self.lock.release()

    # Task utility functions

    def activate_hashtable(self):
        self.lock.acquire()
        try:
            if self.hashtable_running or self.hashtable_stopped:
                return

            self.hashtable_running = 1

            for peer in self.left_neighbours:
                self.probe(self.peers[peer].address)
            for peer in self.right_neighbours:
                if peer not in self.left_neighbours:
                    self.probe(self.peers[peer].address)
        finally:
            self.lock.release()

    def deactivate_hashtable(self):
        """ Once deactivated, the hashtable can not be reactivated. """

        self.lock.acquire()
        try:
            hashtable_was_running = self.hashtable_running
            self.hashtable_running = 0
            self.hashtable_stopped = 1
        finally:
            self.lock.release()

        if not hashtable_was_running:
            return

        def farewell_task(node, address):
            try:
                node.call(address,('going offline',)) # actually means hashtable deactivate
            except Error:
                pass

        farewellers = [ ]
        for peer in self.peers.values():
            if peer.name in self.left_neighbours or \
                 peer.name in self.right_neighbours:
                this_one = Task(farewell_task,self,peer.address)
                farewellers.append(this_one)
                this_one.start()

        def offload_task(node):
            while 1:
                node.lock.acquire()
                try:
                    #print "  %d     \r" % len(node.links),
                    #sys.stdout.flush()
                    if len(node.links) == 0:
                        return

                    #address = node.links[0].dest
                    #links = [ ]
                    #for item in node.links:
                    #  if item.dest == address:
                    #    links.append(item)
                    #for item in links:
                    #  node.links.remove(item)
                    links = node.links[:30]
                    del node.links[:30]

                    if len(node.left_neighbours):
                        left = node.peers[node.left_neighbours[0]].address
                    else:
                        left = None
                finally:
                    node.lock.release()

                #for link in links:
                try:
                    try:
                        if left == None:
                            raise Error('no left neighbour')

                        glob = ['glob']
                        for link in links:
                            glob.append(('store link for',link.name,link.dest,int(link.timeout[1] - time.time())))

                        #node.call(left,('store link for',link.name,link.dest,int(link.timeout[1] - time.time())))
                        node.call(left,tuple(glob), 0,2)
                    except Error, error:
		        pass

			#Everyone has upgrade by now, so can get rid of:
                        #for link in links:
                        #    node.call(address_demangle(link.dest,node.address),('dropped link',link.name), 0,2)
                except Error:
                    #break
                    pass

        def after_farewellers_task(self, farewellers=farewellers,offload_task=offload_task):
            for task in farewellers:
                task.join()

            for i in range(20):
                this_one = Task(offload_task,self)
                self.hashtable_offloaders.append(this_one)
                this_one.start()

        this_one = Task(after_farewellers_task,self)
        self.hashtable_offloaders.append(this_one)
        this_one.start()
 
    def query_peers(self, name,op,size):
        list = [ ]

        self.lock.acquire()
        for peer in self.peers.values():
            if op == '<' :
                list.append((long_name_minus(name,peer.name), peer.name))
            else:
                list.append((long_name_minus(peer.name,name), peer.name))
        self.lock.release()

        def sorter(a,b):
            return long_name_compare(a[0],b[0])

        list.sort(sorter)

        result = [ ]
        for item in list[0:size]:
            result.append(item[1])
         
        return result

    def socket_handle_errors(self):
        try:
            self.socket_lock.acquire()
            try:
                result, address = self.socket.recvfrom(max_packet_size,0x2000 | socket.MSG_DONTWAIT)
            finally:
                self.socket_lock.release()

            state = self.link_states.get(address)
            if state:
                state.error_received()
        except socket.error:
            pass
    
    def socket_sendto(self,msg,address):
        if self.socket_proxy:
            self.socket_proxy.sendto(msg,address)
        else:
            for i in range(100):
                try:
                    self.socket_lock.acquire()
                    try:
                        self.socket.sendto(msg,address)
                        return
                    finally:
                        self.socket_lock.release()
                except socket.error, error:
                    #print error
                    self.socket_handle_errors()
            raise Error('Socket misbehaving.')

    def socket_recvfrom(self):
        if self.socket_proxy:
            is_error, address, message = self.socket_proxy.recvfrom()
            if not is_error:
                return message, address

            self.lock.acquire()
            if self.link_states.has_key(address):
                self.link_states[address].error_received()
            self.lock.release()
            raise Error('socket misbehaving')
        else:
            try:
                self.socket_lock.acquire()
                try:
                    message, address = self.socket.recvfrom(max_packet_size,_recv_flags)
                finally:
                    self.socket_lock.release()
            except socket.error, error:
                #print error
                self.socket_handle_errors()
                raise Error('socket misbehaving')

            return message, address

    def call(self, address, query, low_priority=0, n_retries=udp_retries):
        """Perform a UDP remote procedure call, and return the result.

             This will raise an Error("no reply") if the address can not be contacted.
             It will also raise an Error if the remote computer replies with an
             Error object."""

        return self.call_timed(address,query,low_priority,n_retries)[0]
    
    def call_timed(self, address, query, low_priority=0, n_retries=udp_retries):
        """As in call, but returns a tuple of the result and the time taken once access was
             granted by the link state object."""

        if address == self.address:
            self.lock.acquire()
            ticket = self.call_no
            self.call_no = self.call_no + 1
            self.lock.release()

            reply = handle_request(self,query,address,ticket)
            if type(reply) == types.InstanceType and reply.__class__ == Error:
                raise reply
            return (reply, 0.0)

        #print query[0]
    
        self.lock.acquire()
        try:
            if not self.running:
                raise Error('Node is stopped.')
            
            ticket = self.call_no
            self.call_no = self.call_no + 1

            event = threading.Event()
            self.pending_calls[ticket] = event

            if self.hashtable_running:
                message = (self.name,ticket,query)
            else:
                message = (None,ticket,query)

            if self.link_states.has_key(address):
                state = self.link_states[address]
            else:
                state = Link_state()
                self.link_states[address] = state
            message = safe_dumps(message)
        finally:
            self.lock.release()

        wait_time = state.acquire_access(event,low_priority)

        # link is down
        if wait_time == None:
            raise Error('no reply')
                    
        try:
            start = time.time()
            total_traffic = 0

            # in case socket_sendto fails
            end = start

            success = 0
            for i in range(n_retries):
                if state.is_error():
                    break
                
                try:
                    self.socket_sendto(message, address)
                    total_traffic = total_traffic + len(message)
                    self.lock.acquire()
                    self.network_usage = self.network_usage + len(message)
                    self.lock.release()
                except Error:
                    break

                #event.wait(wait_time)
                for j in range(10):
                    if event.isSet():
                        break
                    time.sleep(wait_time/10.0)

                end = time.time()
 
                self.lock.acquire()
                try:    
                    success = self.replies.has_key(ticket)

                    if success:

                        for j in range(i):
                            self.call_p_success = self.call_p_success*0.99
                        self.call_p_success = self.call_p_success*0.99+0.01
                    
                        total_traffic = total_traffic + self.replies[ticket][1]

                        reply = self.replies[ticket][0]
                        if type(reply) == types.InstanceType and reply.__class__ == Error:
                            raise reply

                        return (reply, end-start)

                    if not self.running:
                        raise Error('node stopped')
                finally:
                    self.lock.release()

        finally:
            state.release_access(event, success,i, end-start, low_priority)
        
            self.lock.acquire()
            del self.pending_calls[ticket]
            if self.replies.has_key(ticket):
                del self.replies[ticket]
            self.lock.release()

            for monitor in self.monitors:
                monitor( 1, address, query, total_traffic )

        if n_retries == udp_retries:
	    state.error_received()

        raise Error('no reply')

    def find_node(self, name, low_priority=0):
        return self.find_nodes(name, 4, low_priority)[0]
        
    def find_nodes(self, name, how_many, low_priority=0):
        failure_list = [ ]
        success_list = [ ]

        if self.hashtable_running:
            list = [(self.name, self.address)]
        else:
            list = self.call(self.address,('get peers','<',name,how_many),low_priority)
            if list == [ ]:
                raise Error('find_nodes failed')
    
        while 1:
            head = list[0]

            try:
                try:
                    returned_value, elapsed_time = self.call_timed(head[1],('glob',('who are you',),('get peers','<',name,how_many)),low_priority)

                    if len(returned_value) <> 2:
                        raise Error('unexpected reply')
                    
                    real_name, result = returned_value

                    #self.ping_total_time = self.ping_total_time + elapsed_time
                    #self.ping_count = self.ping_count + 1
                    if elapsed_time > hashtable_activate_threshhold:
                        self.pings_over_threshhold = self.pings_over_threshhold + 1
                    else:
                        self.pings_under_threshhold = self.pings_under_threshhold + 1
                except Error, error:
                    if error.message == 'no reply':
                        raise error
                    real_name = self.call(head[1],('who are you',),low_priority)
                    result    = self.call(head[1],('get peers','<',name,how_many),low_priority)
                except TypeError:
                    raise Error('bad reply')
                
                if real_name <> head[0]:
                    raise Error('wrong name')

                check_type(result, types.ListType)
                for item in result:
                    check_type(item, types.TupleType)
                    if len(item) <> 2:
                        raise Error('unexpected reply')

                success_list.append(head)
            except Error:
                list = list[1:]
                failure_list.append(head)
                result = [ ]

            for item_mangled in result:
                item = (item_mangled[0],address_demangle(item_mangled[1],head[1]))
                if item not in list and item not in failure_list and item[0] <> self.name:
                    index = 0
                    while index < len(list) and not long_name_bracketed(list[index][0],item[0],name):
                        index = index + 1
                    list.insert(index,item)
            
            if list == [ ]:
                raise Error('find_nodes failed')

            if head == list[0]:
                break

        def fyi_task(self, list, whom):
            address = address_mangle(whom[1],self.address)
            for item in list:
                try:
                    self.call(item[1],('add peer',whom[0],address),1)
                except Error:
                    pass

        # This optimizes interconnections in the network to
        # respond quickly to common queries and in log time
        # to others.
        Task(fyi_task,self,success_list[1:],head).start_no_overload(1)

        return list


